//------------------------------------------------------------------------------
//
//  npglut.c
//
// ANTZ - 3D data viewer with realtime real-world interaction based on NPE.
//
// ANTZ is hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
// Written in 2010-2011 by Shane Saxon - contact@saxondigital.net
//
// Additional code contributed by: Mark Buchanan and Lucas Erickson.
//
// To the extent possible under law, the author(s) have dedicated all copyright 
// and related and neighboring rights to this software to the public domain
// worldwide. This software is distributed without any warranty.
//
// Released under the CC0 license, which is GPL compatible.
//
// You should have received a copy of the CC0 Public Domain Dedication along
// with this software (license file named COPYING.txt). If not, see
// http://creativecommons.org/publicdomain/zero/1.0/
//
//------------------------------------------------------------------------------

#include "npglut.h"

#include "../npio.h"

#include "npgl.h"
#include "npkey.h"
#include "npmouse.h"

#include "../os/npos.h"


#include "../data/npmapfile.h"		//debug, zz


#define kWindowWidth 640
#define kWindowHeight 480
#define kWindowPositionX 40
#define kWindowPositionY 40


void npGlutInitGL();
void npGlutDrawGLScene(void);
void npGlutDrawGLSceneIdle(void);
void npGlutResizeGLScene(int Width, int Height);

void npGlutKeyDown (unsigned char key, int x, int y);
void npGlutKeyUp (unsigned char key, int x, int y);
void npGlutKeyDownSpecial (int key, int x, int y);
void npGlutKeyUpSpecial (int key, int x, int y);


//------------------------------------------------------------------------------
void npInitGlut (int argc, char **argv, void* dataRef)
{

	glutInit (&argc, argv);

//	npLoadTextures (dataRef);

	glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition (kWindowPositionX, kWindowPositionY);
	glutInitWindowSize (kWindowWidth, kWindowHeight);
	glutCreateWindow ("ANTz");
	
//	wglSwapIntervalEXT(1);		//to remove frame tearing with VSYNC debug, zz
//	nposSwapInterval(1);		//move OS methods to npos.h, debug zz
//	glfwSwapInterval(1);		//may use the glfw libarry, not implememted, zz

	//register keyboard events with GLUT
	glutKeyboardFunc (npGlutKeyDown);
	glutKeyboardUpFunc (npGlutKeyUp);
	glutSpecialFunc (npGlutKeyDownSpecial);
	glutSpecialUpFunc (npGlutKeyUpSpecial);
	
	//register mouse events with GLUT
	glutMouseFunc (npMouseEvent);
	glutMotionFunc (npMouseMotion);

	glutDisplayFunc (npGlutDrawGLScene);
	glutIdleFunc (npGlutDrawGLSceneIdle);

	glutReshapeFunc (npGLResizeScene);
	
	//---
	npInitGL (dataRef);

	glutFullScreen ();

	printf ("System Ready...\n\n");
}


//------------------------------------------------------------------------------
void npCloseGlut (void* dataRef)
{
	npCloseGL (dataRef);

	return;
}


//------------------------------------------------------------------------------
void npAppLoopGlut (void* dataRef)
{
	npInitGlut (0, 0, dataRef);
	glutMainLoop();
}


//------------------------------------------------------------------------------
void npGlutDrawGLScene(void) 
{
	int period = 16; //frame period in ms (ie 60fps is 16.66667... approx 16ms
	static long int previousTime = 0;
	long int time = 0;
	long int deltaTime = 0;

	//init time on first run
	if (previousTime == 0)
		previousTime = glutGet(GLUT_ELAPSED_TIME);

	time = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = time - previousTime;
	
	//	printf("deltaTime: %d \n", deltaTime);

	if (deltaTime < period)
		nposSleep( period - deltaTime);			//calls OS specific command

	npGLDrawScene (npGetDataRef());

	glutSwapBuffers();
//	glutPostRedisplay();			//zz helps with forcing redraw, workaround, debug
									//issue with mouse hitting edge of screen
	previousTime = time;
}

//------------------------------------------------------------------------------

void npGlutDrawGLSceneIdle(void)
{
	npGlutDrawGLScene();

	return;
}

//------------------------------------------------------------------------------
void npGlutFullScreenExit()
{
	glutPositionWindow (kWindowPositionX, kWindowPositionY);
	glutReshapeWindow (kWindowWidth, kWindowHeight);        /* Restore us */

//	glutSwapBuffers();
//	glutPostRedisplay();
}


//------------------------------------------------------------------------------
void npGlutKeyDown(unsigned char key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyDown, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyUp(unsigned char key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyUp, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyDownSpecial(int key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyDownSpecial, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyUpSpecial(int key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyUpSpecial, glutGetModifiers() );
}


//------------------------------------------------------------------------------
void npGLSolidSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutSolidSphere (radius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutWireSphere (radius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLSolidCone (GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutSolidCone (base, height, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireCone (GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutWireCone (base, height, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLSolidTorus (GLdouble innerRadius, GLdouble outerRadius, 
					GLint slices, GLint stacks)
{
	glutSolidTorus (innerRadius, outerRadius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireTorus (GLdouble innerRadius, GLdouble outerRadius, 
					GLint slices, GLint stacks)
{
	glutWireTorus (innerRadius, outerRadius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGlutPrimitive (int primitive)
{
	switch (primitive)
	{
	case kNPglutWireCube :
		glScalef (0.6f, 0.6f, 0.6f); 
		glutWireCube(2.0f);
		break;
	case kNPglutSolidCube :
		glScalef (0.6f, 0.6f, 0.6f);
		glutSolidCube(2.0f); 
		break;
	case kNPglutWireSphere : glutWireSphere( 1.0f, 15, 15 ); break;
	case kNPglutSolidSphere : glutSolidSphere( 1.0f, 15, 15 ); break;

	case kNPglutWireCone : glutWireCone( 1.0f, 2.0f, 20, 1 ); break;
	case kNPglutSolidCone : glutSolidCone( 1.0f, 2.0f, 20, 1 ); break;

	case kNPglutWireTorus : glutWireTorus(0.15f, 1.5f, 7, 16); break;
	case kNPglutSolidTorus : glutSolidTorus(0.15f, 1.5f, 7, 16); break;

	case kNPglutWireDodecahedron :
		glScalef (0.6f, 0.6f, 0.6f);
		glutWireDodecahedron(); 
		break;
	case kNPglutSolidDodecahedron :
		glScalef (0.6f, 0.6f, 0.6f);
		glutSolidDodecahedron();
		break;
	case kNPglutWireOctahedron : glutWireOctahedron(); break;
	case kNPglutSolidOctahedron : glutSolidOctahedron(); break;
	case kNPglutWireTetrahedron : glutWireTetrahedron(); break;
	case kNPglutSolidTetrahedron : glutSolidTetrahedron(); break;
	case kNPglutWireIcosahedron : glutWireIcosahedron(); break;
	case kNPglutSolidIcosahedron : glutSolidIcosahedron(); break;

	case kNPglutWireTeapot : glutWireTeapot( 2.0f ); break;
	case kNPglutSolidTeapot : glutSolidTeapot( 2.0f ); break;

	default : glutWireTetrahedron(); break;
	}
}

